package user

import (
	"context"
	"fmt"
	"log"

	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo/options"

	"git.selly.red/Selly-Modules/usermngmt/database"
	"git.selly.red/Selly-Modules/usermngmt/model"
)

func isPhoneNumberExisted(ctx context.Context, phone string) bool {
	var (
		col = database.GetUserCol()
	)
	// Find
	cond := bson.M{
		"phone":   phone,
		"deleted": false,
	}
	total, err := col.CountDocuments(ctx, cond)
	if err != nil {
		return true
	}
	return total != 0
}

func isEmailExisted(ctx context.Context, email string) bool {
	var (
		col = database.GetUserCol()
	)
	// Find
	cond := bson.M{
		"email":   email,
		"deleted": false,
	}
	total, err := col.CountDocuments(ctx, cond)
	if err != nil {
		return true
	}
	return total != 0
}

func isRoleExisted(ctx context.Context, roleID primitive.ObjectID) bool {
	var (
		col = database.GetRoleCol()
	)
	// Find
	cond := bson.M{
		"_id": roleID,
	}
	total, err := col.CountDocuments(ctx, cond)
	if err != nil {
		log.Println("isRoleExisted - count: ", err)
	}
	return total != 0
}

func roleFindByID(ctx context.Context, id primitive.ObjectID) (model.DBRole, error) {
	var (
		doc model.DBRole
		col = database.GetRoleCol()
	)
	err := col.FindOne(ctx, bson.M{"_id": id}).Decode(&doc)
	return doc, err
}

func roleFindByCondition(ctx context.Context, cond interface{}, opts ...*options.FindOptions) (docs []model.DBRole) {
	var (
		col = database.GetRoleCol()
	)
	docs = make([]model.DBRole, 0)

	cursor, err := col.Find(ctx, cond, opts...)
	if err != nil {
		return
	}
	defer cursor.Close(ctx)
	if err = cursor.All(ctx, &docs); err != nil {
		return
	}
	return
}

func permissionFindByCondition(ctx context.Context, cond interface{}, opts ...*options.FindOptions) (docs []model.DBPermission) {
	var (
		col = database.GetPermissionCol()
	)
	docs = make([]model.DBPermission, 0)

	cursor, err := col.Find(ctx, cond, opts...)
	if err != nil {
		return
	}
	defer cursor.Close(ctx)
	if err = cursor.All(ctx, &docs); err != nil {
		return
	}
	return
}

func create(ctx context.Context, doc model.DBUser) error {
	var (
		col = database.GetUserCol()
	)
	_, err := col.InsertOne(ctx, doc)
	if err != nil {
		return fmt.Errorf("error when create user: %s", err.Error())
	}

	return nil
}

func updateOneByCondition(ctx context.Context, cond interface{}, payload interface{}) error {
	var (
		col = database.GetUserCol()
	)
	_, err := col.UpdateOne(ctx, cond, payload)
	if err != nil {
		return fmt.Errorf("error when update user: %s", err.Error())
	}

	return err
}

func updateManyByCondition(ctx context.Context, cond interface{}, payload interface{}) error {
	var (
		col = database.GetUserCol()
	)
	_, err := col.UpdateMany(ctx, cond, payload)
	if err != nil {
		return fmt.Errorf("error when update user: %s", err.Error())
	}

	return err
}

func findByID(ctx context.Context, id primitive.ObjectID) (model.DBUser, error) {
	var (
		doc model.DBUser
		col = database.GetUserCol()
	)
	err := col.FindOne(ctx, bson.M{"_id": id, "deleted": false}).Decode(&doc)
	return doc, err
}

func findByCondition(ctx context.Context, cond interface{}, opts ...*options.FindOptions) (docs []model.DBUser) {
	var (
		col = database.GetUserCol()
	)
	docs = make([]model.DBUser, 0)

	cursor, err := col.Find(ctx, cond, opts...)
	if err != nil {
		return
	}
	defer cursor.Close(ctx)
	if err = cursor.All(ctx, &docs); err != nil {
		return
	}
	return
}

// countByCondition ...
func countByCondition(ctx context.Context, cond interface{}) int64 {
	var (
		col = database.GetUserCol()
	)
	total, err := col.CountDocuments(ctx, cond)
	if err != nil {
		log.Println("user - countByCondition err:", err)
	}
	return total
}

func findOneByCondition(ctx context.Context, cond interface{}) (model.DBUser, error) {
	var (
		col = database.GetUserCol()
		doc model.DBUser
	)
	err := col.FindOne(ctx, cond).Decode(&doc)
	return doc, err
}
